unit Editing;

interface

uses
  SysUtils, SparsePopulationEUC;

function kanjiCode(line: AnsiString): Integer;
procedure editRecord(normal: AnsiString; substitute: AnsiString; binaryName: String; whichTable: Boolean);
procedure editManyRecords(sjis_text: String; binaryName: String; whichTable: Boolean);

type
  NormalSubstitute=record
  please_check: Boolean;
  normal: AnsiString;
  substitute: AnsiString;
end;

const
  BEGINNING_OF_FILE = 0;
  C_SJIS_EUC_BIN = 'C:\diyo\MS932~EUC-JP.bin';
  DATA_ARRAY_LIMIT = 906;
  END_OF_FILE = 2;
  FIRST_KANJI = 159;
  H_0D = 13;
  H_81 = 129;
  H_A1 = 161;
  H_DF = 223;
  H_FC = 252;
  RIGHT_ARROW = '  '; { Shift_JIS }
  SJIS_EUC_BIN = 'MS932~EUC-JP.bin';

var
  data: array[1..DATA_ARRAY_LIMIT] of NormalSubstitute;
  SJIS_EUC_TABLES: array[1..2] of String;

implementation

function kanjiCode(line: AnsiString): Integer;
var
  buffer: PAnsiChar;
  firstByte: Integer;
  i: Integer;
  k: Integer;
  kanji: Boolean;
  n: Integer;
  size: Integer;
  size_1: Integer;

begin
  firstByte := 32;
  kanji := False;

  try
    size := Length(line);
    buffer := System.AllocMem( size + 1 );
{    buffer := AllocMem( size + 1 );  Delphi 4 }
    StrPCopy(buffer, line);

    size_1 := size - 1;
    for i := 0 to size_1 do
    begin
      k := Ord(buffer[i]);

      if kanji then
        begin
          n := k + firstByte;
          break;
        end;

      if (H_81 <= k) and (k <= H_FC) then
        begin
          if (H_A1 <= k) and (k <= H_DF) then
            begin
              n := k;
              break;
            end
          else
            begin
              firstByte := k shl 8;
              kanji := True;
            end;

        end
      else
        begin
          n := k;
          break;
        end;

    end; { for i := 0 to size_1 do }

  finally
    FreeMem(buffer);
  end;

  Result := n;
end;

procedure editRecord(normal: AnsiString; substitute: AnsiString; binaryName: String; whichTable: Boolean);
var
  a: AnsiChar;
  b: Integer;
  binaryFile: TextFile;
  buffer: PAnsiChar;
  bytes: Integer;
  c: AnsiChar;
  d: AnsiChar;
  from :Integer;
  handle: Integer;
  i: Integer;
  into: Integer;
  k: Integer;
  n: Integer;
  recordNumber: Integer;
  size: Integer;
  represented_in_hexadecimal: String;
  table: SparseEUC;

begin
  table := SparseEUC.Create;
  table.setting(whichTable);

  a := Chr(32);
  buffer := @a;

  SJIS_EUC_TABLES[1] := C_SJIS_EUC_BIN;
  SJIS_EUC_TABLES[2] := SJIS_EUC_BIN;

  from := kanjiCode(normal);
  represented_in_hexadecimal := IntToHex(from, 4) + RIGHT_ARROW;

  into := kanjiCode(substitute);
  represented_in_hexadecimal := represented_in_hexadecimal + IntToHex(into, 4);
  WriteLn(represented_in_hexadecimal);

  for n := 1 to 2 do
  begin
    if FileExists(SJIS_EUC_TABLES[n]) then
      begin
        try
          handle := FileOpen(SJIS_EUC_TABLES[n], fmOpenRead);
          size := FileSeek(handle, 0, END_OF_FILE);
          FileSeek(handle, 0, BEGINNING_OF_FILE);
          buffer := System.AllocMem( size + 1 );
{          buffer := AllocMem( size + 1 );  Delphi 4}
          bytes := FileRead(handle, buffer^, size);
          FileClose(handle);

          AssignFile(binaryFile, binaryName);
          Rewrite(binaryFile);

          recordNumber := 1;
          i := 0;

          while i < bytes do
          begin
            if recordNumber < FIRST_KANJI then
              begin
                c := buffer[i]; i := i + 1;
                k := Ord(c);

                d := buffer[i]; i := i + 1;
              end
            else
              begin
                c := buffer[i]; i := i + 1;
                b := Ord(c);
                k := b shl 8;

                d := buffer[i]; i := i + 1;
                b := Ord(c);
                k := k + b;
              end;

            Write(binaryFile, c);
            Write(binaryFile, d);
            if  k = from then
              begin
                c := table.working(into, 0);
                Write(binaryFile, c);

                c := table.working(into, 1);
                Write(binaryFile, c);
                i := i + 2;
              end
            else
              begin
                c := buffer[i]; i := i + 1;
                Write(binaryFile, c);

                c := buffer[i]; i := i + 1;
                Write(binaryFile, c);
              end;

            recordNumber := recordNumber + 1;
          end; { while i < bytes do }

          CloseFile(binaryFile);
        finally
          FreeMem(buffer);
        end;

        break;
      end
    else
      begin
        if n = 2 then
          begin
            table.Free;
            Halt(1);
          end;

      end;

  end; { for n := 1 to 2 do }

  table.Free;
end;

procedure editManyRecords(sjis_text: String; binaryName: String; whichTable: Boolean);
var
  a: AnsiChar;
  b: Integer;
  binaryFile: TextFile;
  buffer: PAnsiChar;
  bytes: Integer;
  c: AnsiChar;
  checked: Boolean;
  d: AnsiChar;
  from: Integer;
  handle: Integer;
  i: Integer;
  into: Integer;
  k: Integer;
  line: String;
  m: Integer;
  n: Integer;
  recordNumber: Integer;
  size: Integer;
  sjisFile: Textfile;
  table: SparseEUC;
  used: Integer;

begin
  checked := False;
  m := 1;
  used := 1;

  AssignFile(sjisFile, sjis_text);
  Reset(sjisFile);

  while not EOF(sjisFile) do
  begin
    ReadLn(sjisFile, line);
    size := Length(line);
    if size = 4 then
      begin
        data[m].please_check := True;
        data[m].normal := Copy(line, 1, 2);
        data[m].substitute := Copy(line, 3, 2);
      end
    else
      begin
        data[m].please_check := True;
        data[m].normal := Copy(line, 1, 1); { katakana, for example }
        data[m].substitute := Copy(line, 2, 1);
      end;

    used := m;
    m := m + 1;
    if m > DATA_ARRAY_LIMIT then
      break;

  end; { while not EOF(sjisFile) do }
  CloseFile(sjisFile);

  table := SparseEUC.Create;
  table.setting(whichTable);

  a := Chr(32);
  buffer := @a;

  SJIS_EUC_TABLES[1] := C_SJIS_EUC_BIN;
  SJIS_EUC_TABLES[2] := SJIS_EUC_BIN;

  for n := 1 to 2 do
  begin
    if FileExists(SJIS_EUC_TABLES[n]) then
      begin
        try
          handle := FileOpen(SJIS_EUC_TABLES[n], fmOpenRead);
          size := FileSeek(handle, 0, END_OF_FILE);
          FileSeek(handle, 0, BEGINNING_OF_FILE);
          buffer := System.AllocMem( size + 1 );
{          buffer := AllocMem( size + 1 );  Delphi 4}
          bytes := FileRead(handle, buffer^, size);
          FileClose(handle);

          AssignFile(binaryFile, binaryName);
          Rewrite(binaryFile);

          recordNumber := 1;
          i := 0;

          while i < bytes do
          begin
            if recordNumber < FIRST_KANJI then
              begin
                c := buffer[i]; i := i + 1;
                k := Ord(c);

                d := buffer[i]; i := i + 1;
              end
            else { recordNumber >= FIRST_KANJI }
              begin
                c := buffer[i]; i := i + 1;
                b := Ord(c);
                k := b shl 8;

                d := buffer[i]; i := i + 1;
                b := Ord(d);
                k := k + b;
              end;

            Write(binaryFile, c);
            Write(binaryFile, d);

            for m := 1 to used do
            begin
              if data[m].please_check then
                begin
                  from := kanjiCode(data[m].normal);
                  if  k = from then
                    begin
                      into := kanjiCode(data[m].substitute);
                      checked := True;
                      data[m].please_check := False;
                      break;
                    end
                  else
                    begin
                      checked := False;
                    end;

                end
              else
                begin
                  checked := False;
                end;

            end; { for m := 1 to used do }

            if checked then
              begin
                c := table.working(into, 0);
                Write(binaryFile, c);

                c := table.working(into, 1);
                Write(binaryFile, c);
                i := i + 2;
              end
            else
              begin
                c := buffer[i]; i := i + 1;
                Write(binaryFile, c);

                c := buffer[i]; i := i + 1;
                Write(binaryFile, c);
              end;

            recordNumber := recordNumber + 1;
          end; { while i < bytes do }

          CloseFile(binaryFile);
        finally
          FreeMem(buffer);
        end;

        break;
      end
    else
      begin
        if n = 2 then
          begin
            table.Free;
            Halt(1);
          end;

      end;

  end; { for n := 1 to 2 do }

  table.Free;
end;

end.
