unit RecordForm;

interface

uses
  Variants, 
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, SparsePopulationEUC, SparsePopulationSJIS, Clipbrd;

type
  TForm3 = class(TForm)
    Edit1: TEdit;
    Edit2: TEdit;
    Button1: TButton;
    Label1: TLabel;
    procedure Button1Click(Sender: TObject);
    procedure FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormCreate(Sender: TObject);
  private
    statusBarText: String;
    procedure editRecord;
    procedure memo;
    procedure testKanji;
  public
    function kanjiCode(line: AnsiString): Integer;
    procedure saveButtenFocused;
  end;

{*
 : 32 (20h)
_: 95 (5Fh)
@: 33088 (8140h)
Q: 33105 (8151h)
*}

const
  BEGINNING_OF_FILE = 0;
  C_SJIS_EUC_BIN = 'C:\SideAceDelphi\MS932~EUC-JP.bin';
  END_OF_FILE = 2;
  FIRST_KANJI = 159;
  HANKAKU_KASEN = 95;
  HANKAKU_KUHAKU = 32;
  H_0D = 13;
  H_81 = 129;
  H_A1 = 161;
  H_DF = 223;
  H_FC = 252;
  MS932 = '@'; { Shift_JIS }
  RECORD_HEIGHT = 130;
  RECORD_WIDTH = 131;
  RIGHT_ARROW = '  ';
  SJIS_EUC_BIN = 'MS932~EUC-JP.bin';
  TEMPORARY_TEXT = 'text.tmp';
  ZENKAKU_KUHAKU = 33088;

var
  Form3: TForm3;
  SJIS_EUC_TABLES: array[1..2] of String;

implementation

{$R *.dfm}

function TForm3.kanjiCode(line: AnsiString): Integer;
var
  buffer: PAnsiChar;
  firstByte: Integer;
  i: Integer;
  k: Integer;
  kanji: Boolean;
  n: Integer;
  size: Integer;
  size_1: Integer;

begin
  firstByte := 32;
  kanji := False;

  try
    size := Length(line);
    buffer := System.AllocMem( size + 1 );
{    buffer := AllocMem( size + 1 );  Delphi 4 }
    StrPCopy(buffer, line);

    size_1 := size - 1;
    for i := 0 to size_1 do
    begin
      k := Ord(buffer[i]);

      if kanji then
        begin
          n := k + firstByte;
          break;
        end;

      if (H_81 <= k) and (k <= H_FC) then
        begin
          if (H_A1 <= k) and (k <= H_DF) then
            begin
              n := k;
              break;
            end
          else
            begin
              firstByte := k shl 8;
              kanji := True;
            end;

        end
      else
        begin
          n := k;
          break;
        end;

    end; { for i := 0 to size_1 do }

  finally
    FreeMem(buffer);
  end;

  Result := n;
end;

procedure TForm3.editRecord;
var
  a: AnsiChar;
  b: Integer;
  binaryFile: TextFile;
  buffer: PAnsiChar;
  bytes: Integer;
  c: AnsiChar;
  d: AnsiChar;
  from :Integer;
  handle: Integer;
  i: Integer;
  into: Integer;
  k: Integer;
  n: Integer;
  recordNumber: Integer;
  size: Integer;
  table: SparseEUC;
  whichTable: Boolean;

begin
  whichTable := False;
  table := SparseEUC.Create;
  table.setting(whichTable);

  a := Chr(32);
  buffer := @a;

  SJIS_EUC_TABLES[1] := C_SJIS_EUC_BIN;
  SJIS_EUC_TABLES[2] := SJIS_EUC_BIN;

  from := kanjiCode(Edit1.Text);
  statusBarText := IntToHex(from, 4) + RIGHT_ARROW;

  into := kanjiCode(Edit2.Text);
  statusBarText := statusBarText + IntToHex(into, 4);
  Label1.Caption := statusBarText;

  for n := 1 to 2 do
  begin
    if FileExists(SJIS_EUC_TABLES[n]) then
      begin
        try
          handle := FileOpen(SJIS_EUC_TABLES[n], fmOpenRead);
          size := FileSeek(handle, 0, END_OF_FILE);
          FileSeek(handle, 0, BEGINNING_OF_FILE);
          buffer := System.AllocMem( size + 1 );
{          buffer := AllocMem( size + 1 );  Delphi 4}
          bytes := FileRead(handle, buffer^, size);
          FileClose(handle);

          AssignFile(binaryFile, SJIS_EUC_TABLES[n]);
          Rewrite(binaryFile);

          recordNumber := 1;
          i := 0;

          while i < bytes do
          begin
            if recordNumber < FIRST_KANJI then
              begin
                c := buffer[i]; i := i + 1;
                k := Ord(c);

                d := buffer[i]; i := i + 1;
              end
            else
              begin
                c := buffer[i]; i := i + 1;
                b := Ord(c);
                k := b shl 8;

                d := buffer[i]; i := i + 1;
                b := Ord(c);
                k := k + b;
              end;

            Write(binaryFile, c);
            Write(binaryFile, d);
            if  k = from then
              begin
                c := table.working(into, 0);
                Write(binaryFile, c);

                c := table.working(into, 1);
                Write(binaryFile, c);
                i := i + 2;
              end
            else
              begin
                c := buffer[i]; i := i + 1;
                Write(binaryFile, c);

                c := buffer[i]; i := i + 1;
                Write(binaryFile, c);
              end;

            recordNumber := recordNumber + 1;
          end; { while i < bytes do }

          CloseFile(binaryFile);
        finally
          FreeMem(buffer);
        end;

        break;
      end
    else
      begin
        if n = 2 then
          begin
            table.Free;
            Halt(1);
          end;

      end;

  end; { for n := 1 to 2 do }

  table.Free;
end;

procedure TForm3.memo;
var
  m: array[0..16] of WideChar;
  p: PWideChar;
begin
  p := @m;
  StringToWideChar(statusBarText, p, 17);
  Clipboard.SetTextBuf(p);
end;

{*
procedure TForm3.memo;
var
  m: array[0..16] of Char;
  p: PChar;
begin
  p := @m;
  StrPCopy(p, statusBarText);
  Clipboard.SetTextBuf(p);
end;  Delphi 4
*}
procedure TForm3.saveButtenFocused;
begin
  Button1.SetFocus;
end;

procedure TForm3.testKanji;
var
  c: AnsiChar;
  k: Integer;
  line: String;
  table: SparseSJIS;
  temporaryFile: TextFile;
  whichTable: Boolean;

begin
  whichTable := False;
  table := SparseSJIS.Create;
  table.setting(whichTable);

  k := kanjiCode(Edit1.Text);
  statusBarText := IntToHex(k, 4);
  Label1.Caption := statusBarText;

  AssignFile(temporaryFile, TEMPORARY_TEXT);
  Rewrite(temporaryFile);

  if k < ZENKAKU_KUHAKU then
    begin
      c := table.working(k, 0);
      Write(temporaryFile, c);
    end
  else
    begin
      c := table.working(k, 0);
      Write(temporaryFile, c);

      c := table.working(k, 1);
      Write(temporaryFile, c);
    end;

  CloseFile(temporaryFile);

  Reset(temporaryFile);
  ReadLn(temporaryFile, line);
  CloseFile(temporaryFile);

  Edit2.Text := line;
  table.Free;
end;

procedure TForm3.FormCreate(Sender: TObject);
begin
  BorderIcons := [biSystemMenu];
  Left := ( Screen.Width - RECORD_WIDTH ) div 2;
  Top := ( Screen.Height - RECORD_HEIGHT ) div 2;
  Width := RECORD_WIDTH;
  Height := RECORD_HEIGHT;
  Edit1.Text := MS932;
  Label1.Height := 24;
  Label1.Width := 127;

  testKanji;
end;

procedure TForm3.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if Key = VK_ESCAPE then
    Close;

  if Key = VK_HOME then
    testKanji;

  if Key = VK_F8 then
    memo;

end;

procedure TForm3.Button1Click(Sender: TObject);
begin
{ Save button }
  editRecord;
end;

end.
