unit SideAceForm;

interface

{*
SideAce Delphi Version 1.06
Displaying the SideAce window
Copyright 2019 Sogaya

Delphi 4
Delphi 2009

*}

uses
  Variants,  
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, Menus, Math, SparsePopulationEUC, SparsePopulationUTF8, ShellAPI,
  ExtCtrls;

type
  TForm1 = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Leave1: TMenuItem;
    Exit1: TMenuItem;
    Encode1: TMenuItem;
    EUCJP1: TMenuItem;
    ShiftJIS1: TMenuItem;
    UTF81: TMenuItem;
    Help1: TMenuItem;
    Large1: TMenuItem;
    Standard1: TMenuItem;
    Version1: TMenuItem;
    Memo1: TMemo;
    StatusBar1: TStatusBar;
    Button1: TButton;
    Timer1: TTimer;
    Timer2: TTimer;
    Record1: TMenuItem;
    Timer3: TTimer;
    procedure Version1Click(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure Leave1Click(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure EUCJP1Click(Sender: TObject);
    procedure ShiftJIS1Click(Sender: TObject);
    procedure UTF81Click(Sender: TObject);
    procedure Large1Click(Sender: TObject);
    procedure Standard1Click(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure Timer2Timer(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure StatusBar1Click(Sender: TObject);
    procedure FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure Record1Click(Sender: TObject);
    procedure Timer3Timer(Sender: TObject);
  private
    ASCII_KATAKANA: AnsiString;
    corner: Integer;
    count: Integer;
    fullName: Boolean;
    header: Integer;
    leaveACE: Boolean;
    lineList: TStrings;
    manager: Integer;
    NEW_LINE: AnsiString;
    renewal: Boolean;
    statusBarText: AnsiString;
    touched: Integer;
    upperLimit: Integer;
    function columns(x: Integer): Integer;
    function fixedWidthFontScreenLinePlus(line: AnsiString; limit: Integer): AnsiString;
    function readACE: Boolean;
    function rows(x: Integer): Integer;
    procedure convert_to_EUC;
    procedure convert_to_UTF8;
    procedure keepLog; { Shift_JIS }
    procedure largeHandler;
    procedure moveToCorner;
    procedure seePDF;
    procedure setting;
    procedure statusBarMessage(s: String);
  public
    { Public  }
  end;

const
  ACE = 'ACE';
  ASCII = ' !"#$%&()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_`abcdefghijklmnopqrstuvwxyz{|}~';
  ASCII_SIZE = 1;
  BORDER = 38.0;
  BOTTOM_ROW = 26.0;
  BREAK_1 = 2147483646; { 2147483647 - 1 }
  CORNER_0 = '-c';
  CORNER_1 = '-c1';
  CORNER_2 = '-c2';
  CORNER_3 = '-c3';
  CORNER_4 = '-c4';
  EMPTY = '';
  ENCODING_1 = '-e1';
  ENCODING_2 = '-e2';
  ENCODING_3 = '-e3';
  ERROR_TEXT = 'error.txt';
  EUC_JP = 'EUC-JP';
  FULL_NAME = '-f';
  GAP = ' ';
  HEAD = 130.0;
  HYPHEN = '-';
  H_0D = 13;
  H_81 = 129;
  H_A1 = 161;
  H_DF = 223;
  H_FC = 252;
  KANJI_SIZE = 2;
  KATAKANA = ''; { Shift_JIS }
  MAIN_BORDER = 639.0; { MAIN_WIDTH - BORDER }
  MAIN_HEAD = 361.0; { MAIN_HEIGHT - HEAD }
  MAIN_HEIGHT = 491;
  MAIN_WIDTH = 677;
  MANY_TIMES = 30;
  MANY_TIMES_1 = MANY_TIMES - 1;
  NONSAMPLING_PERIOD = 300; { milliseconds }
  PDF = '-p';
  PDF_FILE_NAME = 'SideAceDelphi.pdf';
  PLUS = '+';
  PLUS_SIZE = 1;
  SETTING_PHASE = 70; { milliseconds }
  SHIFT_JIS = 'Shift_JIS';
  SIDE_ACE = 'SideAce';
  SIDE_ACE_DELPHI = 'SideAce Delphi';
  SLEEPING = 200; { milliseconds }
  TASK_BAR_HEIGHT = 43;
  TO_EUC = 1;
  TO_SJIS = 2;
  TO_UTF8 = 3;
  UPPER_LIMIT = 80.0;
  UTF_8 = 'UTF-8';
  VERSION_NUMBER = '1.06';

var
  Form1: TForm1;

implementation

uses VersionForm, RecordForm;

{$R *.dfm}

procedure TForm1.Button1Click(Sender: TObject);
begin
  largeHandler;
end;

function TForm1.columns(x: Integer): Integer;
var
  m_BORDER: Double;
  x_BORDER: Double;
  y: Double;
begin
  m_BORDER := MAIN_BORDER;
  x_BORDER := x - BORDER;
  y := x_BORDER * UPPER_LIMIT/m_BORDER;
  Result := Floor(y);
end;

procedure TForm1.convert_to_EUC;
var
  a: AnsiChar;
  buffer: PAnsiChar;
  c: AnsiString;
  COUNT_1: Integer;
  firstByte: Integer;
  line_number_1: Integer;
  i: Integer;
  k: Integer;
  kanji: Boolean;
  line: AnsiString;
  size: Integer;
  size_1: Integer;
  table: SparseEUC;
  eucFile: TextFile;
  whichTable: Boolean;

begin
  whichTable := False;
  table := SparseEUC.Create;
  table.setting(whichTable);

  a := Chr(32);
  buffer := @a;
  c := EMPTY;
  COUNT_1 := lineList.Count - 1;
  firstByte := 32;
  kanji := False;

  AssignFile(eucFile, ACE);
  Rewrite(eucFile);

  for line_number_1 := 0 to COUNT_1 do
  begin
    try
      line := lineList[ line_number_1 ] + Chr(10); { 0Ah }
      size := Length(line);
      buffer := System.AllocMem( size + 1 );
{      buffer := AllocMem( size + 1 );  Delphi 4}
      StrPCopy(buffer, line);

      size_1 := size - 1;
      for i := 0 to size_1 do
      begin
        a := buffer[i];
        k := Ord(buffer[i]);

        if kanji then
          begin
            k := k + firstByte;

            c := table.working(k, 0);
            Write(eucFile, c);
            c := table.working(k, 1);
            Write(eucFile, c);

            kanji := False;
            continue;
          end;

        if (H_81 <= k) and (k <= H_FC) then
          begin
            if (H_A1 <= k) and (k <= H_DF) then
              begin
                c := table.working(k, 0);
                Write(eucFile, c);

                kanji := False;
              end
            else
              begin
                firstByte := k shl 8;
                kanji := True;
              end;

          end
        else
          begin
            Write(eucFile, a);
            kanji := False;
          end;

      end; { for i := 0 to size_1 do }

    finally
      FreeMem(buffer);
    end;

  end; { for line_number_1 := 0 to COUNT_1 do }

  CloseFile(eucFile);
  table.Free;
end;

procedure TForm1.convert_to_UTF8;
var
  a: AnsiChar;
  buffer: PAnsiChar;
  c: AnsiString;
  COUNT_1: Integer;
  firstByte: Integer;
  line_number_1: Integer;
  i: Integer;
  k: Integer;
  kanji: Boolean;
  line: AnsiString;
  size: Integer;
  size_1: Integer;
  table: SparseUTF8;
  utf8File: TextFile;
  whichTable: Boolean;

begin
  whichTable := False;
  table := SparseUTF8.Create;
  table.setting(whichTable);

  a := Chr(32);
  buffer := @a;
  c := EMPTY;
  COUNT_1 := lineList.Count - 1;
  firstByte := 32;
  kanji := False;

  AssignFile(utf8File, ACE);
  Rewrite(utf8File);

  for line_number_1 := 0 to COUNT_1 do
  begin
    try
      line := lineList[ line_number_1 ] + Chr(10);
      size := Length(line);
      buffer := System.AllocMem( size + 1 );
{      buffer := AllocMem( size + 1 );  Delphi 4}
      StrPCopy(buffer, line);

      size_1 := size - 1;
      for i := 0 to size_1 do
      begin
        a := buffer[i];
        k := Ord(buffer[i]);

        if kanji then
          begin
            k := k + firstByte;

            c := table.working(k, 0);
            Write(utf8File, c);
            c := table.working(k, 1);
            Write(utf8File, c);
            c := table.working(k, 2);
            Write(utf8File, c);

            kanji := False;
            continue;
          end;

        if (H_81 <= k) and (k <= H_FC) then
          begin
            if (H_A1 <= k) and (k <= H_DF) then
              begin
                c := table.working(k, 0);
                Write(utf8File, c);
                c := table.working(k, 1);
                Write(utf8File, c);
                c := table.working(k, 2);
                Write(utf8File, c);

                kanji := False;
              end
            else
              begin
                firstByte := k shl 8;
                kanji := True;
              end;

          end
        else
          begin
            Write(utf8File, a);
            kanji := False;
          end;

      end; { for i := 0 to size_1 do }

    finally
      FreeMem(buffer);
    end;

  end; { for line_number_1 := 0 to COUNT_1 do }

  CloseFile(utf8File);
  table.Free;
end;

function TForm1.fixedWidthFontScreenLinePlus(line: AnsiString; limit: Integer): AnsiString;
var
  a: AnsiChar;
  buffer: PAnsiChar;
  c: AnsiString;
  k: Integer;
  q: Integer;
  sequence: AnsiString;
  size: Integer;
begin
  a := Chr(32);
  buffer := @a;
  c := EMPTY;
  k := 0;
  sequence := EMPTY;

  try
    size := Length(line);
    buffer := System.AllocMem( size + 1 );
{    buffer := AllocMem( size + 1 );  Delphi 4}
    StrPCopy(buffer, line);

    while k < size do
    begin
      c := buffer[k];
      q := AnsiPos(c, ASCII_KATAKANA);
      if q >= 1 then
        begin
          sequence := sequence + c;
          k := k + ASCII_SIZE;
        end
      else
        begin
          sequence := sequence + c;

          c := buffer[ k + 1 ];
          sequence := sequence + c;

          k := k + KANJI_SIZE;
        end;

      if k >= limit then
        begin
          sequence := sequence + PLUS;
          break;
        end;

    end; { while k < size do }

  finally
    FreeMem(buffer);
  end;

  Result := sequence;
end;

procedure TForm1.keepLog;
begin
  lineList.SaveToFile(ACE);
end;

procedure TForm1.largeHandler;
begin
  WindowState := wsNormal;
  Left := 0;
  Top := 0;
  Width := Screen.Width;
  Height := Screen.Height - TASK_BAR_HEIGHT;
end;

procedure TForm1.moveToCorner;
begin
  case corner of
    0: begin
         Left := ( Screen.Width - Width ) div 2;
         Top := ( Screen.Height - Height ) div 2;
       end;
    1: begin
         Left := 0;
         Top := 0;
       end;
    2: begin
         Left := Screen.Width - MAIN_WIDTH;
         Top := 0;
       end;
    3: begin
         Left := Screen.Width - MAIN_WIDTH;
         Top := Screen.Height - MAIN_HEIGHT - TASK_BAR_HEIGHT;
       end;
    4: begin
         Left := 0;
         Top := Screen.Height - MAIN_HEIGHT - TASK_BAR_HEIGHT;
       end;
  else { Corner-3 }
    Left := Screen.Width - MAIN_WIDTH;
    Top := Screen.Height - MAIN_HEIGHT - TASK_BAR_HEIGHT;
  end;

end;

function TForm1.readACE: Boolean;
const
  reading = True;
var
  aceFile: TextFile;
  b: Boolean;
  line: AnsiString;
  lineNumber: Integer;
  head: AnsiString;
  s: AnsiString;
  UPPER_LIMIT_1: Integer;
begin
  b := True;

  try
    AssignFile(aceFile, ACE);
    Reset(aceFile);
    lineNumber := 1;
    head := '';
    UPPER_LIMIT_1 := upperLimit - PLUS_SIZE;
    while not EOF(aceFile) do
    begin
      ReadLn(aceFile, line);
      if lineNumber <= header then
        begin
          s := fixedWidthFontScreenLinePlus(line, UPPER_LIMIT_1);
          head := head + s + NEW_LINE;
        end;

      lineList.Add(line);
      lineNumber := lineNumber + 1;
    end;

    CloseFile(aceFile);
    Memo1.Text := head;
  except
    b := False;
  end;

  Result := b;
end;

procedure TForm1.Record1Click(Sender: TObject);
begin
  Form3.Visible := True;
  Form3.saveButtenFocused;
end;

function TForm1.rows(x: Integer): Integer;
var
  m_HEAD: Double;
  x_HEAD: Double;
  y: Double;
begin
  m_HEAD := MAIN_HEAD;
  x_HEAD := x - HEAD;
  y := x_HEAD * BOTTOM_ROW/m_HEAD;
  Result := Floor(y);
end;

procedure TForm1.seePDF;
begin
  ShellExecute(Handle, 'open', PDF_FILE_NAME, nil, nil, SW_NORMAL);
  Halt(1);
end;

procedure TForm1.setting;
var
  aceFile: TextFile;
  c: Char;
  head: String;
  i: Integer;
  s: String;
begin
  Timer3.Enabled := False;

  AssignFile(aceFile, ACE);
  Rewrite(aceFile);
  c := Chr(32); { 20h }
  Write(aceFile, c);
  CloseFile(aceFile);

  lineList := TStringList.Create;
  ASCII_KATAKANA := ASCII + KATAKANA + Chr(27);
  NEW_LINE := Chr(13) + Chr(10);

  leaveACE := False;

  header := rows(Height);
  upperLimit := columns(Width);

  corner := 3;
  fullName := False;
  manager := TO_UTF8;
  for i := 1 to ParamCount do
  begin
    s := ParamStr(i);
    head := Copy(s, 1, 1);
    if CompareStr(head, HYPHEN) = 0 then
      begin
        if CompareStr(s, CORNER_0) = 0 then
          corner := 0;

        if CompareStr(s, CORNER_1) = 0 then
          corner := 1;

        if CompareStr(s, CORNER_2) = 0 then
          corner := 2;

        if CompareStr(s, CORNER_3) = 0 then
          corner := 3;

        if CompareStr(s, CORNER_4) = 0 then
          corner := 4;

        if CompareStr(s, ENCODING_1) = 0 then
          manager := TO_EUC;

        if CompareStr(s, ENCODING_2) = 0 then
          manager := TO_SJIS;

        if CompareStr(s, ENCODING_3) = 0 then
          manager := TO_UTF8;

        if CompareStr(s, FULL_NAME) = 0 then
          fullName := True;

        if CompareStr(s, PDF) = 0 then
          seePDF;

      end
  end;
  moveToCorner;

  if fullName then
    Caption := SIDE_ACE_DELPHI
  else
    Caption := SIDE_ACE;

  touched := FileAge(ACE);
  Timer1.Interval := NONSAMPLING_PERIOD;
  Timer1.Enabled := True;
  Timer2.Enabled := False;
end;

procedure TForm1.statusBarMessage(s: String);
begin
  statusBarText := s;
  StatusBar1.Panels[0].Text := statusBarText;
end;

procedure TForm1.EUCJP1Click(Sender: TObject);
begin
  manager := TO_EUC;
  statusBarMessage(EUC_JP);
end;

procedure TForm1.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  KeyPreview := True;
  Memo1.Text := GAP;
  statusBarMessage( GetCurrentDir );

  Left := Screen.Width - MAIN_WIDTH;
  Top := Screen.Height -MAIN_HEIGHT - TASK_BAR_HEIGHT;
  Height := MAIN_HEIGHT;
  Width := MAIN_WIDTH;
  header := Floor(BOTTOM_ROW);
  upperLimit := Floor(UPPER_LIMIT);

  Button1.Height := 34;
  Button1.Width := 68;
  Button1.Left := ClientWidth - Button1.Width;
  Button1.Top := ClientHeight - Button1.Height - StatusBar1.Height;

  Timer3.Interval := SETTING_PHASE;
  Timer3.Enabled := True;
end;

procedure TForm1.FormDestroy(Sender: TObject);
var
  aceFile: TextFile;
  temporaryFile: TextFile;

begin
  if FileExists(ACE) then
      begin
        AssignFile(aceFile, ACE);
        Erase(aceFile);
      end;

  if FileExists(TEMPORARY_TEXT) then
      begin
        AssignFile(temporaryFile, TEMPORARY_TEXT);
        Erase(temporaryFile);
      end;

  if leaveACE then
    begin
      case manager of
        TO_EUC: convert_to_EUC;
        TO_SJIS: keepLog;
        TO_UTF8: convert_to_UTF8;
      else
        convert_to_UTF8;
      end;

    end;

  lineList.Free;
end;

procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if Key = VK_F3 then
    Close;

  if Key = VK_F8 then
    Memo1.Text := statusBarText;

end;

procedure TForm1.FormResize(Sender: TObject);
begin
  Button1.Left := ClientWidth - Button1.Width;
  Button1.Top := ClientHeight - Button1.Height - StatusBar1.Height;
  upperLimit := columns(Width);
  header := rows(Height);
end;

procedure TForm1.Large1Click(Sender: TObject);
begin
  largeHandler;
end;

procedure TForm1.Leave1Click(Sender: TObject);
begin
  leaveACE := True;
  Close;
end;

procedure TForm1.ShiftJIS1Click(Sender: TObject);
begin
  manager := TO_SJIS;
  statusBarMessage(SHIFT_JIS);
end;

procedure TForm1.Standard1Click(Sender: TObject);
begin
  WindowState := wsNormal;
  Height := MAIN_HEIGHT;
  Width := MAIN_WIDTH;
  moveToCorner;
end;

procedure TForm1.StatusBar1Click(Sender: TObject);
begin
  Memo1.Text := GAP;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
var
  k: Integer;
begin
  k := FileAge(ACE);
  if k <> touched then
    begin
      touched := k;
      Timer1.Enabled := False;

      count := 0;
      Timer2.Interval := SLEEPING;
      Timer2.Enabled := True;
    end;

end;

procedure TForm1.Timer2Timer(Sender: TObject);
var
  errorFile: TextFile;
begin
  count := count + 1;
  if count < MANY_TIMES then
  begin
    renewal := readACE;
    if renewal then
      begin
        Timer2.Enabled := False;

        Timer1.Interval := NONSAMPLING_PERIOD;
        Timer1.Enabled := True;
        count := BREAK_1;
      end
    else
      begin
        if count = MANY_TIMES_1 then
          begin
            Timer2.Enabled := False;

            AssignFile(errorFile, ERROR_TEXT);
            Rewrite(errorFile);
            Write(errorFile, 'count = MANY_TIMES_1');
            CloseFile(errorFile);
            Close;
          end;

      end;

  end;

end;

procedure TForm1.Timer3Timer(Sender: TObject);
begin
  setting;
end;

procedure TForm1.UTF81Click(Sender: TObject);
begin
  manager := TO_UTF8;
  statusBarMessage(UTF_8);
end;

procedure TForm1.Version1Click(Sender: TObject);
begin
  Form2.setVersion(PDF_FILE_NAME, VERSION_NUMBER);
  Form2.Visible := True;
end;

end.
