unit SparsePopulationSJIS;

interface

uses
  SysUtils;

type
  SparseSJIS = class
  private
    s: array[1..193764] of AnsiChar;
{*

FC4Ch = 64588
64588 * ( SJIS_KANJI_SIZE + 1 ) = 193764

*}
  public
    procedure clear;
    procedure Filed(s: String);
    procedure populated(index: Integer; place: Integer; c: AnsiChar);
    procedure setting(whichTable: Boolean);
    function working(index: Integer; place: Integer): AnsiChar;
  end;

const
  C_MS932_SJIS_BIN = 'C:\SideAceDelphi\MS932~Shift_JIS.bin';
  FIRST_KANJI = 159;
  MESSAGE_TEXT = 'message.txt';
  SJIS_KANJI_SIZE = 2;
  SJIS_KANJI_SIZE_1 = 3;
  MS932_SJIS_BIN = 'MS932~Shift_JIS.bin';

var
  MS932_SJIS_TABLES: array[1..2] of String;

implementation

procedure SparseSJIS.populated(index: Integer; place: Integer; c: AnsiChar);
var
  k: Integer;
begin
  k := SJIS_KANJI_SIZE_1 * index + place;
  s[k] := c
end;

procedure SparseSJIS.clear;
var
  k: Integer;
begin
  for k := 1 to 193764 do
    s[k] := #0;

end;

procedure SparseSJIS.Filed(s: String);
var
  messageFile: TextFile;
begin
  AssignFile(messageFile, MESSAGE_TEXT);
  Rewrite(messageFile);
  WriteLn(s);
  CloseFile(messageFile);
end;

procedure SparseSJIS.setting(whichTable: Boolean);
const
  BEGINNING_OF_FILE = 0;
  END_OF_FILE = 2;

var
  a: AnsiChar;
  b: Integer;
  c: AnsiChar;
  C_IS_NOT_USED: AnsiChar;
  buffer: PAnsiChar;
  bytes: Integer;
  handle: Integer;
  i: Integer;
  k: Integer;
  n: Integer;
  recordNumber: Integer;
  size: Integer;

begin
  a := Chr(32);
  buffer := @a;

  MS932_SJIS_TABLES[1] := C_MS932_SJIS_BIN;
  MS932_SJIS_TABLES[2] := MS932_SJIS_BIN;

  for n := 1 to 2 do
  begin
    if FileExists(MS932_SJIS_TABLES[n]) then
      begin
        try
          handle := FileOpen(MS932_SJIS_TABLES[n], fmOpenRead);
          size := FileSeek(handle, 0, END_OF_FILE);
          FileSeek(handle, 0, BEGINNING_OF_FILE);
{          buffer := System.AllocMem( size + 1 );  Delphi 2009 }
          buffer := AllocMem( size + 1 );
          bytes := FileRead(handle, buffer^, size);
          FileClose(handle);

          recordNumber := 1;
          i := 0;

          while i < bytes do
          begin
            if recordNumber < FIRST_KANJI then
              begin
                c := buffer[i]; i := i + 1;
                k := Ord(c);

                C_IS_NOT_USED := buffer[i]; i := i + 1;

                c := buffer[i]; i := i + 1;
                populated(k, 0, c);

                c := buffer[i]; i := i + 1;
                populated(k, 1, c);
              end
            else
              begin
                c := buffer[i]; i := i + 1;
                b := Ord(c);
                k := b shl 8;

                c := buffer[i]; i := i + 1;
                b := Ord(c);
                k := k + b;

                c := buffer[i]; i := i + 1;
                populated(k, 0, c);

                c := buffer[i]; i := i + 1;
                populated(k, 1, c);
              end;

            recordNumber := recordNumber + 1;
          end; { while i < bytes do }

        finally
          FreeMem(buffer);
        end;

        if whichTable then
          Filed( MS932_SJIS_TABLES[n] + ' is primarily accessible to SideAce.' );

        break;
      end
    else
      begin
        if n = 2 then
          begin
            Halt(1);
          end;

      end;

  end;

end;

function SparseSJIS.working(index: Integer; place: Integer): AnsiChar;
var
  k: Integer;
begin
  k := SJIS_KANJI_SIZE_1 * index + place;
  Result := s[k];
end;

end.
